"""
TBD
"""

__author__ = "Lukas Mahler"
__version__ = "1.0.0"
__date__ = "14.04.2022"
__email__ = "m@hler.eu"
__status__ = "Development"


# Default
import sys
import json
import time
import random
import inspect

# Custom
import ts3
import requests

# Self
from src import util


def annoy(self, invoker, parameter):
    """

    """

    if not len(parameter) != 1:
        err = "Please use the command like this: .annoy TARGET MESSAGE"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return

    target = parameter[0]
    msg = parameter[1]
    self.poke(n=10, msg=msg, usr=target)


def follow(self, invoker, parameter):
    """
    TODO sticky folgen
    """

    if not parameter:
        err = "Please use the command like this: .follow TARGET"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return

    target = parameter[0]
    clid = self.identifytarget(target)
    cid = self.bot.clientinfo(clid=int(clid))[0]["cid"]
    self.bot.clientmove(clid=self.myid, cid=cid)


def help(self, invoker):
    """

    """

    # Find all functions in this submodule
    cmds = [f[0] for f in inspect.getmembers(sys.modules[__name__], inspect.isfunction)]
    msg = f"List of commands:\n---------------------\n.{f'{chr(10)}.'.join(cmds)}"
    self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=msg)


def info(self, invoker):
    """

    """

    msg = f"Runtime: {util.getRuntime(self.started)}\n" \
          f"Version: {self.version}"
    self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=msg)


def list(self, invoker, parameter):
    """
    Message the invoker of the function either a list of channels, clients, commands or server groups.
    """

    if not parameter:
        err = "Please use the command like this: .list channel/clients/commands/groups/"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return

    what = parameter[0]
    mydict = {}

    if what == "channel":
        channels = self.bot.channellist()
        for channel in channels:
            order = channel["channel_order"]
            mydict[order] = [channel["channel_name"], channel["cid"]]

        mydict = dict(sorted(mydict.items()))
        msg = json.dumps(mydict)

    elif what == "clients":
        clients = self.bot.clientlist()
        for client in clients:
            mydict[client["client_nickname"]] = {"clid": client["clid"], "cldbid": client["client_database_id"]}
        mydict = dict(sorted(mydict.items()))
        msg = json.dumps(mydict)

    elif what == "commands":
        help(self, invoker)
        return

    elif what == "groups":
        groups = self.bot.servergrouplist()
        for group in groups:
            mydict[group["name"]] = {"sgid": group["sgid"], "sortid": group["sortid"]}
        msg = json.dumps(mydict)

    else:
        msg = f"The parameter [{what}] is not supported."

    self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=msg)


def mute(self, invoker, parameter):
    """
    Assign the mute group to a user.
    """

    if not parameter:
        err = "Please use the command like this: .mute TARGET"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return

    target = parameter[0]
    clid = self.identifytarget(target)
    cldbid = self.clid2cldbid(clid)

    try:
        self.bot.servergroupaddclient(sgid=self.sgid_mute, cldbid=cldbid)
    except ts3.query.TS3QueryError as e:
        err = f"Failed to add to group: [{e}]"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return


def pingall(self, invoker, parameter):
    """

    """

    if not parameter:
        err = "Please use the command like this: .pingall MESSAGE"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return

    msg = parameter[0]
    self.poke(msg=msg)


def notifyAdmin(self):
    """
    Message every available admin.
    """

    clients = self.bot.clientlist()
    clients = [client for client in clients if client["client_type"] != "1"]
    for client in clients:
        cldbid = client["client_database_id"]
        clid = client["clid"]
        if self.isadmin(cldbid):
            msg = "<Keep this chat open to use commands>"
            self.bot.sendtextmessage(targetmode=1, target=clid, msg=msg)

        if not self.whitelisted:
            time.sleep(1)


def quit(self, invoker):
    """

    """
    self.stop(invoker)


def rename(self, invoker, parameter):
    """
    Rename the Bot to the given Nickname
    """

    if not parameter:
        err = "Please use the command like this: .rename NICKNAME"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return

    self.nickname = parameter[0]
    self.bot.clientupdate(client_nickname=self.nickname)


def rmb(self, invoker, parameter):
    """
    Convert rmb (Chinese Yuan) to eur (Euro)
    """

    if not parameter:
        err = "Please use the command like this: .rmb AMOUNT"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return

    url = "https://www.floatrates.com/daily/cny.json"
    req = requests.get(url)

    if req.status_code != 200:
        self.pipeOut(req.status_code, lvl="WARNING")
        return

    data = json.loads(req.text)

    if not data:
        return

    if "eur" not in data:
        self.pipeOut(data, lvl="WARNING")
        return

    try:
        conversion_rate = float(data['eur']['inverseRate'])
    except KeyError:
        conversion_rate = 6.94  # Static

    if "k" in parameter[0]:
        rmb_val = float(parameter[0].replace("k", "000"))
    else:
        rmb_val = float(parameter[0])

    eur = rmb_val / conversion_rate
    msg = f"[{rmb_val:.0f}] rmb is approx. [{eur:.0f}] eur / conversion rate [{conversion_rate:.2f}]"

    self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=msg)


def roll(self, invoker):

    cid_invoker = self.bot.clientinfo(clid=int(invoker['id']))[0]["cid"]
    cid_self = self.bot.clientinfo(clid=self.myid)[0]["cid"]

    if cid_self != cid_invoker:
        self.bot.clientmove(clid=self.myid, cid=cid_invoker)

    rolled = random.randint(0, 100)
    msg = f"Rolled a [{rolled:3d}] by {invoker['name']}"
    self.bot.sendtextmessage(targetmode=2, target=cid_invoker, msg=msg)


def rwf(self):
    """
    Refresh the WoW progress channel,
    this is very specific to my use, you can disable this in the config.
    """
    url = "https://raider.io/api/v1/raiding/progression?raid=sepulcher-of-the-first-ones&difficulty=mythic&region=world"
    req = requests.get(url)

    if req.status_code != 200:
        self.pipeOut(req.status_code, lvl="WARNING")
        return

    data = json.loads(req.text)

    if not data:
        return

    if "progression" not in data:
        self.pipeOut(data, lvl="WARNING")
        return

    prog_done = 0
    prog_max = len(data['progression'])

    for i in range(0, prog_max):
        if data['progression'][i]['totalGuilds'] > 0:
            prog_done = i

    guilds_done = data['progression'][prog_done]['totalGuilds']

    channelname = f"[cspacerC3]Sepulcher {prog_done+1}/{prog_max} ({guilds_done})"

    try:
        self.editChannelname(205, channelname)
    except ts3.query.TS3QueryError:
        pass


def ticker(self, parameter):
    """
    Refresh the crypto Ticker channels,
    this is very specific to my use, you can disable this in the config.
    """

    if parameter:
        symbol = parameter[0].lower()
    else:
        symbol = "all"

    if symbol in ["btc", "bitcoin", "eth", "ethereum", "all"]:
        channelname = f"[cspacerC1]BTC: {self.gecko.getSymbol('bitcoin', decimal=0)} | " \
                      f"ETH: {self.gecko.getSymbol('ethereum', decimal=0)}"
        try:
            self.editChannelname(200, channelname)
        except ts3.query.TS3QueryError:
            pass

    if symbol in ["dot", "polkadot", "ada", "cardano", "all"]:
        channelname = f"[cspacerC2]DOT: {self.gecko.getSymbol('polkadot', decimal=2)} | " \
                      f"ADA: {self.gecko.getSymbol('cardano', decimal=2)}"
        try:
            self.editChannelname(201, channelname)
        except ts3.query.TS3QueryError:
            pass


def unmute(self, invoker, parameter):
    """
    Remove the mute group to a user.
    """

    if not parameter:
        err = "Please use the command like this: .unmute TARGET"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return

    target = parameter[0]
    clid = self.identifytarget(target)
    cldbid = self.clid2cldbid(clid)

    try:
        self.bot.servergroupdelclient(sgid=self.sgid_mute, cldbid=cldbid)
    except ts3.query.TS3QueryError as e:
        err = f"Failed to remove from group: [{e}]"
        self.bot.sendtextmessage(targetmode=1, target=invoker['id'], msg=err)
        return


if __name__ == "__main__":
    exit()
